from .location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
from .county_ml import CountyMl
from .point import Point
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)  # noqa 501


class County(GenericCRUD):
    county_ml = CountyMl()

    def __init__(self):
        logger.start("start init County")

        super().__init__(default_schema_name=LocationLocalConstants.LOCATION_SCHEMA_NAME,  # noqa501
                         default_table_name=LocationLocalConstants.COUNTY_TABLE_NAME,  # noqa501
                         default_view_table_name=LocationLocalConstants.COUNTY_VIEW_NAME,  # noqa501
                         default_id_column_name=LocationLocalConstants.COUNTY_ID_COLUMN_NAME)  # noqa501

        logger.end("end init County")

    def insert(
            self, county: str, lang_code: LangCode,
            title_approved: bool = False, coordinate: Point = None,
            group_id: int = None) -> int:
        logger.start("start insert county",
                     object={'coordinate': coordinate, 'county': county,
                     'lang_code': lang_code, 'title_approved': title_approved})  # noqa501

        county_json = {'coordinate': coordinate,
                       'group_id': group_id}

        county_id = super().insert(data_json=county_json)

        county_ml_id = self.county_ml.insert(county_id=county_id,
                                             county=county,
                                             lang_code=lang_code,
                                             title_approved=title_approved)

        logger.end("end insert county",
                   object={'county_id': county_id,
                           'county_ml_id': county_ml_id})
        return county_id

    def read(self, location_id: int):
        logger.start("start read location",
                     object={'location_id': location_id})
        result = super().select_one_dict_by_id(id_column_value=location_id,
                                                select_clause_value=LocationLocalConstants.COUNTY_TABLE_COLUMNS)  # noqa501

        logger.end("end read location",
                   object={"result": result})  # noqa 501
        return result

    @staticmethod
    def get_county_id_by_county_name_state_id(county_name: str, state_id: int = None) -> int:  # noqa501
        logger.start("start get_county_id_by_county_name_state_id",
                     object={'county_name': county_name, 'state_id': state_id})


        county_id_json = County.county_ml.select_one_dict_by_where(select_clause_value=LocationLocalConstants.COUNTY_ID_COLUMN_NAME,  # noqa501
                                                                                     where=f"title='{county_name}'")  # noqa501
        county_id = county_id_json.get(LocationLocalConstants.COUNTY_ID_COLUMN_NAME)  # noqa501

        logger.end("end get_county_id_by_county_name_state_id",
                   object={'county_id': county_id})
        return county_id



from .location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
from .neighborhood_ml import NeighborhoodMl
from .point import Point
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)  # noqa501


class Neighborhood(GenericCRUD):
    neighborhood_ml = NeighborhoodMl()

    def __init__(self):
        logger.start("start init Neighborhood")
        super().__init__(default_schema_name=LocationLocalConstants.LOCATION_SCHEMA_NAME,  # noqa501
                         default_table_name=LocationLocalConstants.NEIGHBORHOOD_TABLE_NAME,  # noqa501
                         default_view_table_name=LocationLocalConstants.NEIGHBORHOOD_VIEW_NAME,  # noqa501
                         default_id_column_name=LocationLocalConstants.NEIGHBORHOOD_ID_COLUMN_NAME)  # noqa501

        logger.end("end init Neighborhood")

    def insert(
            self, coordinate: Point,
            neighborhood: str, lang_code: LangCode,
            title_approved: bool = False, city_id: int = None,
            group_id: int = None) -> int:

        logger.start("start insert neighborhood",
                     object={'coordinate': coordinate,
                             'neighborhood': neighborhood,
                             'lang_code': lang_code,
                             'title_approved': title_approved,
                             'city_id': city_id,
                             'group_id': group_id})
        neighborhood_json = {
                key: value for key, value in {
                    'coordinate': coordinate,
                    'city_id': city_id,
                    'group_id': group_id
                }.items() if value is not None
            }
        neighborhood_id = super().insert(data_json=neighborhood_json)

        neighborhood_ml_id = self.neighborhood_ml.insert(neighborhood=neighborhood, neighborhood_id=neighborhood_id,  # noqa501
                                                         lang_code=lang_code, title_approved=title_approved)  # noqa501

        logger.end("end insert neighborhood",
                   object={'neighborhood_id': neighborhood_id,
                           'neighborhood_ml_id': neighborhood_ml_id})
        return neighborhood_id

    def read(self, location_id: int):
        logger.start("start read location",
                     object={'location_id': location_id})
        result = super().select_one_dict_by_id(id_column_value=location_id,
                                                select_clause_value=LocationLocalConstants.NEIGHBORHOOD_TABLE_COLUMNS)  # noqa501

        logger.end("end read location",
                   object={"result": result})  # noqa 501
        return result

    @staticmethod
    def get_neighborhood_id_by_neighborhood_name(neighborhood_name: str,
                                                 city_id: int = None) -> int:
        logger.start("start get_neighborhood_id_by_neighborhood_name",
                     object={'neighborhood_name': neighborhood_name,
                             'city_id': city_id})

        neighborhood_id_json = Neighborhood.neighborhood_ml.select_one_dict_by_where(select_clause_value=LocationLocalConstants.NEIGHBORHOOD_ID_COLUMN_NAME,  # noqa501
                                                                                     where=f"title='{neighborhood_name}'")  # noqa501
        neighborhood_id = neighborhood_id_json.get(LocationLocalConstants.NEIGHBORHOOD_ID_COLUMN_NAME)  # noqa501

        logger.end("end get_neighborhood_id_by_neighborhood_name",
                   object={'neighborhood_id': neighborhood_id})
        return neighborhood_id



from .location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud_ml import GenericCRUDML   # noqa
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)  # noqa 501


class NeighborhoodMl(GenericCRUDML):

    def __init__(self):
        logger.start("start init NeighborhoodMl")
        super().__init__(default_schema_name=LocationLocalConstants.LOCATION_SCHEMA_NAME,  # noqa501
                         default_table_name=LocationLocalConstants.NEIGHBORHOOD_ML_TABLE_NAME,  # noqa501
                         default_view_table_name=LocationLocalConstants.NEIGHBORHOOD_ML_VIEW_NAME,  # noqa501
                         default_id_column_name=LocationLocalConstants.NEIGHBORHOOD_ML_ID_COLUMN_NAME)  # noqa501
        logger.end("end init NeighborhoodMl")

    def insert(self, neighborhood_id: int, neighborhood: str,
               lang_code: LangCode, title_approved: bool = False):
        logger.start("start insert neighborhood_ml",
                     object={'neighborhood_id': neighborhood_id,
                             'neighborhood': neighborhood,
                             'lang_code': lang_code,
                             'title_approved': title_approved})
        neighborhood_ml_json = {
                key: value for key, value in {
                    'neighborhood_id': neighborhood_id,
                    'lang_code': lang_code,
                    'title': neighborhood,
                    'title_approved': title_approved
                }.items() if value is not None
            }
        neighborhood_ml_id = super().insert(data_json=neighborhood_ml_json)
        logger.end("end insert neighborhood_ml",
                   object={'neighborhood_ml_id': neighborhood_ml_id})

        return neighborhood_ml_id



import os
from typing import Dict
from opencage.geocoder import OpenCageGeocode
from .location_local_constants import LocationLocalConstants
from .country_ml import CountryMl
from .point import Point
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402
api_key = os.getenv("OPENCAGE_KEY")

logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)  # noqa501


class Country(GenericCRUD):
    country_ml = CountryMl()

    def __init__(self):
        logger.start("start init Country")

        super().__init__(default_schema_name=LocationLocalConstants.LOCATION_SCHEMA_NAME,  # noqa501
                         default_table_name=LocationLocalConstants.COUNTRY_TABLE_NAME,  # noqa501
                         default_view_table_name=LocationLocalConstants.COUNTRY_VIEW_NAME,  # noqa501
                         default_id_column_name=LocationLocalConstants.COUNTRY_ID_COLUMN_NAME)  # noqa501

        logger.end("end init Country")

    def insert(self, country: str, lang_code: LangCode,
               title_approved: bool = False,
               new_country_data: Dict[str, any] = None,
               coordinate: Point = None) -> int:
        logger.start("start insert country",
                     object={'coordinate': coordinate, 'country': country,
                             'lang_code': lang_code,
                             'title_approved': title_approved,
                             'new_country_data': new_country_data})

        new_country_data = new_country_data or {}
        try:
            country_json = {
                key: value for key, value in {
                    'coordinate': coordinate,
                    'iso': new_country_data.get("iso"),
                    'name': country,
                    'nicename': new_country_data.get("nicename"),
                    'iso3': new_country_data.get("iso3"),
                    'numcode': new_country_data.get("numcode"),
                    'phonecode': new_country_data.get("phonecode")
                }.items() if value is not None
            }
            country_id = super().insert(data_json=country_json)

        except Exception as e:
            logger.exception("error in insert country")
            logger.end()
            raise e
        try:
            country_ml_id = self.country_ml.insert(country=country,
                                                   country_id=country_id,
                                                   lang_code=lang_code,
                                                   title_approved=title_approved)  # noqa501
        except Exception as e:
            logger.exception("error in insert country")
            logger.end()
            raise e
        logger.end("end insert country",
                   object={'country_id': country_id,
                           'country_ml_id': country_ml_id})
        return country_id

    def read(self, location_id: int):
        logger.start("start read location",
                     object={'location_id': location_id})
        result = super().select_one_dict_by_id(id_column_value=location_id,
                                                select_clause_value=LocationLocalConstants.COUNTRY_TABLE_COLUMNS)  # noqa501

        logger.end("end read location",
                   object={"result": result})  # noqa 501
        return result

    @staticmethod
    def get_country_id_by_country_name(country_name: str) -> int:
        logger.start("start get_country_id_by_country_name",
                     object={'country_name': country_name})

        country_id_json = Country.country_ml.select_one_dict_by_where(select_clause_value=LocationLocalConstants.COUNTRY_ID_COLUMN_NAME,  # noqa501
                                                                                     where=f"title='{country_name}'")  # noqa501
        country_id = country_id_json.get(LocationLocalConstants.COUNTRY_ID_COLUMN_NAME)  # noqa501

        logger.end("end get_country_id_by_country_name",
                   object={'country_id': country_id})
        return country_id

    @staticmethod
    def get_country_name(location):
        # Create a geocoder instance
        logger.start("start get_country_name",
                     object={'location': location})

        # Define the city or state
        geocoder = OpenCageGeocode(api_key)

        # Use geocoding to get the location details
        results = geocoder.geocode(location)

        if results and len(results) > 0:
            first_result = results[0]
            components = first_result['components']

            # Extract the country from components
            country_name = components.get('country', '')
            if not country_name:
                # If country is not found, check for country_code as an alternative  # noqa501
                country_name = components.get('country_code', '')
        else:
            country_name = None
            logger.error("country didnt  found for %s." % location)
        logger.end("end get_country_name",
                   object={'country_name': country_name})
        return country_name



from .location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
from .state_ml import StateMl
from .point import Point
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)  # noqa501


class State(GenericCRUD):
    state_ml = StateMl()

    def __init__(self):
        logger.start("start init State")

        super().__init__(default_schema_name=LocationLocalConstants.LOCATION_SCHEMA_NAME,  # noqa501
                         default_table_name=LocationLocalConstants.STATE_TABLE_NAME,  # noqa501
                         default_view_table_name=LocationLocalConstants.STATE_VIEW_NAME,  # noqa501
                         default_id_column_name=LocationLocalConstants.STATE_ID_COLUMN_NAME)  # noqa501
        logger.end("End init State")

    def insert(
            self, coordinate: Point,
            state: str, lang_code: LangCode, state_name_approved: bool = False,
            country_id: int = None, group_id: int = None) -> int:
        logger.start("start insert state",
                     object={'coordinate': coordinate, 'state': state,
                             'lang_code': lang_code,
                             'state_name_approved': state_name_approved,
                             'country_id': country_id, 'group_id': group_id})

        state_json = {
                key: value for key, value in {
                    'coordinate': coordinate,
                    'group_id': group_id
                }.items() if value is not None
            }

        state_id = super().insert(data_json=state_json)

        state_ml_id = self.state_ml.insert(state_id=state_id, state=state,
                                           lang_code=lang_code,
                                           title_approved=state_name_approved)

        logger.end("end insert state",
                   object={'state_id': state_id,
                           'state_ml_id': state_ml_id})
        return state_id

    def read(self, location_id: int):
        logger.start("start read location",
                     object={'location_id': location_id})
        result = super().select_one_dict_by_id(id_column_value=location_id,
                                                select_clause_value=LocationLocalConstants.STATE_TABLE_COLUMNS)  # noqa501

        logger.end("end read location",
                   object={"result": result})  # noqa 501
        return result

    @staticmethod
    def get_state_id_by_state_name(state_name: str,
                                   country_id: int = None) -> int:
        logger.start("start get_state_id_by_state_name",
                     object={'state_name': state_name, 'country_id': country_id})  # noqa501

        state_id_json = State.state_ml.select_one_dict_by_where(select_clause_value=LocationLocalConstants.STATE_ID_COLUMN_NAME,  # noqa501
                                                                                     where=f"state_name='{state_name}'")  # noqa501
        state_id = state_id_json.get(LocationLocalConstants.STATE_ID_COLUMN_NAME)  # noqa501

        logger.end("end get_state_id_by_state_name",
                   object={'state_ids': state_id})
        return state_id



from circles_local_database_python.to_sql_interface import ToSQLInterface


class Point(ToSQLInterface):
    def __init__(self, longitude, latitude):
        self.longitude = longitude
        self.latitude = latitude

    def to_sql(self):
        return f"POINT ({self.longitude}, {self.latitude})"
